﻿using LoongCAD.Geometry;
using TargetComponentType = LoongCAD.STC.Object3D.CustomComponent;

namespace BoxBeamS13
{
    /// <summary>
    /// 组件上下文类，该类对象用来存储组件公用变量、属性
    /// </summary>
    public class ComponentInstanceContext
    {
        /// <summary>
        /// 细部组件实例
        /// </summary>
        public TargetComponentType TargetComp { get => ConnectComp; set => ConnectComp = value; }
        public ParamsHandler ParamsHandler { get; set; }

        #region 组件内部自定义 
        public Vec3 CtrlPt0 { get; private set; }
        public Vec3 CtrlPt1 { get; private set; }
        #endregion

        #region 组件内部自定义
        /// <summary>
        /// 公用ParamNames的数据
        /// </summary>
        public TargetComponentType ConnectComp;

        #endregion

        public static ComponentInstanceContext CreatInstance(TargetComponentType comp, ParamsHandler handler)
        {
            var compContext = new ComponentInstanceContext();
            if (compContext.Initialize(comp, handler))
                return compContext;
            compContext.Clear();
            return null;
        }

        //具体的组件实例
        private bool Initialize(TargetComponentType comp, ParamsHandler handler)
        {
            if (!InitCustomProps(comp, handler)) { return false; }
            return true;
        }
        private bool InitCustomProps(TargetComponentType comp, ParamsHandler handler)
        {

            if (handler == null)
            {
                return false;
            }
            ParamsHandler = handler;
            if (comp == null)
            {
                return false;
            }
            //把传过来的组件，放到上下文
            TargetComp = comp;
            var data = TargetComp.CustomProps;

            if (data == null)
            {
                return false;
            }

            if (TargetComp.CtrlPoints == null) { return false; }
            if (TargetComp.CtrlPoints.Length < 1) { return false; }

            CtrlPt0 = TargetComp.CtrlPoints[0].WorldPos;
            CtrlPt1 = TargetComp.CtrlPoints[1].WorldPos;

            return true;
        }


        private void Clear()
        {
            CtrlPt0 = new Vec3();
            CtrlPt1 = new Vec3();
        }

    }
}