﻿using LoongCAD.Geometry;
using LoongCAD.STC;
using LoongCAD.STC.Object3D;
using LoongCAD.STC.Plugins;
using LoongCAD.UI;
using LoongCAD.UI.Component;
using LoongCAD.Util;
using System;
using System.Windows.Forms;
using TargetComponentType = LoongCAD.STC.Object3D.CustomComponent;

namespace BoxBeamS13
{
    public class PluginMain : CustomCompPlugin
    {
        private readonly Guid m_nameGuid = new Guid(System.Reflection.Assembly.GetExecutingAssembly().GetName().Name);
        /// <summary>
        /// 用于开启组件定义基于代码，而非基于软件
        /// </summary>
        public override bool StandAlone => true;

        public override Guid Guid => m_nameGuid;

        public override int CompNumber => 004;

        public override string CompName => "自定义组件";

        public override string CompKeyword => "";

        public override string CompDescription => "";

        public override string CompShortDescription => "";

        public override string CompGroup => "";

        public override Controller CreateController => new CreateStairController(this);


        public PluginMain() { }

        private readonly Form_PluginMain m_propsForm = null;
        private readonly Form_PluginMain m_defaultForm;

        public override bool OnInstanceCreate(ComponentBase instObj, CompCreateInfo cinfo)
        {
            //处理相对坐标
            var data = cinfo.UserData;
            //设置组件图标位置
            var ctrlPos0 = data.GetVec3(CreateStairController.CTRL_POS_0);
            var ctrlPos1 = data.GetVec3(CreateStairController.CTRL_POS_1);
            instObj.ObjectWorldMatrix = new Mat43(Vec3.AXIS_X, Vec3.AXIS_Y, Vec3.AXIS_Z, ctrlPos0);
            instObj.SetWorldCtrlPoints(new[] { ctrlPos0, ctrlPos1 });
            return base.OnInstanceCreate(instObj, cinfo);
        }

        public override bool OnInstanceRun(ComponentBase compInst, long defaultId)
        {
            if (!(compInst is TargetComponentType inst))
                return false;

            //1. 获取组件实例的属性
            DObject compProps = inst.UIProps;
            if (compProps == null)
            {
                compProps = new DObject();
            }
            //2. 初始化参数处理器，此过程包含以下操作：
            //* 初始化参数值
            //* 初始化参数空值表
            //* 初始化参数内置默认值表
            //* 组合参数默认值
            //其他代码可以直接复制该段代码
            ParamsHandler paramsHandler = new ParamsHandler();
            paramsHandler.ParamValues = compProps;
            paramsHandler.InitializeDefaultValues(GetDefaultsData(defaultId));

            var compContext = ComponentInstanceContext.CreatInstance(inst, paramsHandler);
            if (compContext == null)
            {
                return false;
            }

            //将参数处理器赋值给上下文
            compContext.ParamsHandler = paramsHandler;
            return true;
        }

        // 显示插件的窗口， 未创建的插件
        public override bool OnShowPluginPropsForm(Form ownerForm)
        {

            Form_ComponentProps pluginForm = GetMainForm(ownerForm);
            pluginForm.ShowPluginProps();
            return true;
        }

        // 显示插件的窗口， 已创建的插件 
        public override bool OnShowInstancePropsForm(ComponentBase instObj, long defId, Form ownerForm)
        {
            if (instObj == null)
                return false;

            Form_ComponentProps pluginForm = GetMainForm(ownerForm);
            pluginForm.ShowInstanceProps(instObj,
                new ParamsHandler().InitializeDefaultValues(GetDefaultsData(defId)));
            return true;
        }

        private Form_PluginMain GetMainForm(Form ownerForm)
        {
            Form_PluginMain pluginForm = FormPool.GetPluginForm(Guid, "main") as Form_PluginMain;
            if (pluginForm == null)
            {
                pluginForm = new Form_PluginMain(this);
                pluginForm.Owner = ownerForm;
                FormPool.PushPluginForm(Guid, "main", pluginForm);
            }

            return pluginForm;
        }

    }
}
